#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_GET_AC  0
#define RQ_GET_P 1
#define RQ_GET_AC2 2
#define RQ_GET_P2 3

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
    static uint8_t switch_state2;
    static uint16_t val_ac;
    static uint16_t val_ac2;

    if (rq->bRequest == RQ_GET_AC)
    {
        ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
        ADMUX |= PC0;   // ตั้ง MUX ให้ชี้ไปยัง channel        
        ADCSRA = 0b11000110;
        while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
        val_ac = (ADCL + ADCH*256);
        usbMsgPtr = (uchar*) &val_ac;
        return sizeof(val_ac);
        //return sizeof(5);
    }

    else if(rq->bRequest == RQ_GET_AC2)        
    {
        ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
        ADMUX |= PC1;   // ตั้ง MUX ให้ชี้ไปยัง channel
        ADCSRA = 0b11000110;
        while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
        val_ac2 = (ADCL + ADCH*256);
        usbMsgPtr = (uchar*) &val_ac2;
        return sizeof(val_ac2);
        //return sizeof(5);
    }

    else if(rq->bRequest == RQ_GET_P)
	{
        if ((PINC & (1<<PC4)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);	
	}

    else if(rq->bRequest == RQ_GET_P2)
	{
        if ((PINC & (1<<PC3)) == 0) /* switch is pressed */
            switch_state2 = 1;
        else
            switch_state2 = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state2;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state2);	
	}

    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  = (1<<PC2) | (1<<PC5); // PC2 PC5 = Output, PC0 PC1 PC3 PC4 = Input
    PORTC = (1 << PC4) | (1<<PC3);                 // Enable pull-up at PC2

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(100);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
